using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;


namespace CSysVars{
	/// <summary>
	/// Zusammendfassende Beschreibung fr Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form{
    // Men
    private System.Windows.Forms.MainMenu mainMenu1;
    // Men Datei
    private System.Windows.Forms.MenuItem m_menuFile;
    private System.Windows.Forms.MenuItem m_menuFileLoad;
    private System.Windows.Forms.MenuItem m_menuFileSave;
    private System.Windows.Forms.MenuItem m_menuFileSep1;
    private System.Windows.Forms.MenuItem m_menuFileSaveXSD;
    private System.Windows.Forms.MenuItem m_menuFileSep2;
    private System.Windows.Forms.MenuItem m_menuFileQuit;
    // Men Bearbeiten
    private System.Windows.Forms.MenuItem m_menuEdit;
    private System.Windows.Forms.MenuItem m_menuEditAdoptmarked;
    private System.Windows.Forms.MenuItem m_menuEditAdoptall;
    private System.Windows.Forms.MenuItem m_menuEditSep1;
    private System.Windows.Forms.MenuItem m_menuEditCopy;
    private System.Windows.Forms.MenuItem m_menuEditPaste;
    //Men Ansicht
    private System.Windows.Forms.MenuItem m_menuView;
    private System.Windows.Forms.MenuItem m_menuViewDiffonly;
    private System.Windows.Forms.MenuItem m_menuViewNodims;
    //Men Hilfe
    private System.Windows.Forms.MenuItem m_menuHelp;
    private System.Windows.Forms.MenuItem m_menuHelpContents;
    private System.Windows.Forms.MenuItem m_menuHelpSep1;
    private System.Windows.Forms.MenuItem m_menuHelpAbout;


    private System.Windows.Forms.ListView lv;
    private System.Windows.Forms.ColumnHeader chVarname;
    private System.Windows.Forms.ColumnHeader chLeft;
    private System.Windows.Forms.ColumnHeader chRight;
    private System.Windows.Forms.ColumnHeader chType;
    private System.Windows.Forms.ColumnHeader chDescr;
    private System.Windows.Forms.OpenFileDialog openFileDialog1;
    private System.Windows.Forms.SaveFileDialog saveFileDialog1;
    private System.Windows.Forms.StatusBar statusBar;
		/// <summary>
		/// Erforderliche Designervariable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// Enthlt die gesamten Daten zu den Systemvariablen
		/// </summary>
 	  internal ArrayList m_sysVars = new ArrayList();
		/// <summary>
		/// Aktuelle Zeichnungsdatenbank
		/// </summary>
    internal Database m_db = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Database;
		/// <summary>
		/// Zeigt an, ob ein Vergleich geladen wurde
		/// </summary>
    internal bool m_compareLoaded = false;
		/// <summary>
		/// Steuert die Anzeige von Bemaungsvariablen
		/// </summary>
    internal bool m_showDimVars = true;
		/// <summary>
		/// Nur unterschiedliche anzeigen oder alle
		/// </summary>
    internal bool m_showOnlyDiffs = false;
		/// <summary>
		/// Enthlt die XSD-Validierung fr den Import
		/// </summary>
    internal MemoryStream m_xsdValidationStream;

		/// <summary>
		/// Ctor Hauptdialog
		/// </summary>
    public Form1(){
			//
			// Erforderlich fr die Windows Form-Designeruntersttzung
			//
			InitializeComponent();

			//
			// TODO: Fgen Sie den Konstruktorcode nach dem Aufruf von InitializeComponent hinzu
			//
      readActVars();
      createXsdValidationStream();
      displayData();
    }

		/// <summary>
		/// Erzeugt die XSD-Validierung
		/// </summary>
    internal void createXsdValidationStream(){
      m_xsdValidationStream = XmlUtils.createXsdValidationStream(m_sysVars);
    }


		/// <summary>
		/// Aktualisiert die Anzeige der Daten im Dialog
		/// </summary>
    private void displayData(){
      if(m_compareLoaded == false){
        displayDataLeft();
      }else{
        displayDataAll();
      }
      lv.FindForm().Text = "CSysVars - " + m_db.Filename; 
    }

		/// <summary>
		/// Aktualisiert die Anzeige der Daten im Dialog (kein Vergleich geladen)
		/// </summary>
    private void displayDataLeft(){
      lv.BeginUpdate();
      lv.Items.Clear();

      for (int i=0; i < m_sysVars.Count; i++){
        SysVarData itemData = (SysVarData)m_sysVars[i];

        if((m_showDimVars == true) || (itemData.IsDimVar() == false)){
          // Spalte 0: Label (Variablenname) 
          ListViewItem lvItem = new ListViewItem(itemData.Label);
          // Spalte 1: Aktueller (linker) Wert
          lvItem.SubItems.Add(itemData.StrValueL());
          // Spalte 2: Vergleichswert
          lvItem.SubItems.Add("");
          // Spalte 3: Datentyp
          lvItem.SubItems.Add(itemData.typeStr());
          // Spalte 4: Beschreibung
          lvItem.SubItems.Add(itemData.Descr);
          // Tag (wofr?)
          lvItem.Tag = itemData;
          // Reinpfeifen
          lv.Items.Add(lvItem);
        }
      }
      
      lv.EndUpdate();
    }

		/// <summary>
		/// Aktualisiert die Anzeige der Daten im Dialog (Vergleich geladen)
		/// </summary>
    private void displayDataAll(){
      lv.BeginUpdate();
      lv.Items.Clear();

      for (int i=0; i < m_sysVars.Count; i++){
        SysVarData itemData = (SysVarData)m_sysVars[i];

        if((m_showDimVars == true) || (itemData.IsDimVar() == false)){
          // Spalte 0: Label (Variablenname) 
          ListViewItem lvItem = new ListViewItem(itemData.Label);
          // Spalte 1: Aktueller (linker) Wert
          lvItem.SubItems.Add(itemData.StrValueL());
          // Spalte 2: Vergleichswert
          lvItem.SubItems.Add(itemData.StrValueR());
          // Spalte 3: Datentyp
          if(! itemData.EqualValues()){
            lvItem.SubItems[0].ForeColor = Color.Red;
          }
          lvItem.SubItems.Add(itemData.typeStr());
          // Spalte 4: Beschreibung
          lvItem.SubItems.Add(itemData.Descr);
          // Tag (wofr?)
          lvItem.Tag = itemData;
          // Reinpfeifen
          lv.Items.Add(lvItem);
        }
      }
      
      lv.EndUpdate();
    }

		/// <summary>
		/// List die Systemvariablen in der aktuellen Zeichnung aus
		/// </summary>
    private void readActVars(){
      string str = "";
      int i = 0;
      double d = 0;
      Point2d p2d = new Point2d(0, 0);
      Point3d p3d = new Point3d(0, 0, 0);
      Vector2d v2d = new Vector2d(0, 0);
      Vector3d v3d = new Vector3d(0, 0, 0);

      // DIMFIT veraltet
      // DIMSHO veraltet
      // DIMUNIT veraltet
      m_sysVars.Add(new SysVarDataDouble("ANGBASE", false, "Setzt den Basiswinkel bezogen auf das aktuelle BKS auf 0", m_db.Angbase));
      m_sysVars.Add(new SysVarDataBool("ANGDIR", false, "Stellt die Richtung positiver Winkel ein (0 = GUZ, 1 = UZ)", m_db.Angdir));
      m_sysVars.Add(new SysVarDataInt("ATTMODE", false, "Steuert die Anzeige von Attributen (0 = Aus, 1 = Normal, 2 = Ein)", m_db.Attmode));
      m_sysVars.Add(new SysVarDataInt("AUNITS", false, "Bestimmt die Winkeleinheiten (0 = DezGrad, 1 = GradMinSek, 2 = Neugrad, 3 = Bogenma, 4 Feldma)", m_db.Aunits));
      m_sysVars.Add(new SysVarDataInt("AUPREC", false, "Bestimmt die Anzahl angezeigten Dezimalstellen fr Winkel", m_db.Auprec));
      m_sysVars.Add(new SysVarDataString("CECOLOR", false, "Legt die Farbe fr neue Objekte fest (BYLAYER, BYBLOCK und Ganzzahlen zwischen 1 und 255 oder TrueColor-Werte)", m_db.Cecolor.ToString()));
      m_sysVars.Add(new SysVarDataDouble("CELTSCALE", false, "Legt den Linientyp-Skalierfaktor relativ zu LTSCALE fr neue Objekte fest", m_db.Celtscale));
      m_sysVars.Add(new SysVarDataString("CELTYPE", false, "Legt den Linientyp fr neue Objekte fest", Utils.SymbolIdToName(m_db.Celtype)));
      m_sysVars.Add(new SysVarDataString("CELWEIGHT", false, "Legt die Linienstrke fr neue Objekte fest (-1 = VONLAYER, -2 = VONBLOCK, -3 = VORGABE oder Angaben in 1/100mm)", m_db.Celweight.ToString()));
      m_sysVars.Add(new SysVarDataDouble("CHAMFERA", false, "Legt den ersten Fasenabstand fest", m_db.Chamfera));
      m_sysVars.Add(new SysVarDataDouble("CHAMFERB", false, "Legt den zweiten Fasenabstand fest", m_db.Chamferb));
      m_sysVars.Add(new SysVarDataDouble("CHAMFERC", false, "Legt die Fasenlnge fest", m_db.Chamferc));
      m_sysVars.Add(new SysVarDataDouble("CHAMFERD", false, "Legt den Fasenwinkel fest", m_db.Chamferd));
      m_sysVars.Add(new SysVarDataString("CLAYER", false, "Legt den aktuellen Layer fest", Utils.SymbolIdToName(m_db.Clayer)));
      m_sysVars.Add(new SysVarDataInt("CMLJUST", false, "Legt die Ausrichtung fr Multilinien fest(0 = Oben, 1 = Mitte, 2 = Unten)", m_db.Cmljust));
      m_sysVars.Add(new SysVarDataDouble("CMLSCALE", false, "Steuert die Gesamtbreite einer Multilinie. Bei einem negativen Skalierfaktor wird die Reihenfolge der Abstandslinien umgekehrt", m_db.Cmlscale));
      
      // Hier geht's erstmal ber Interop, da das kein Table-Objekt ist
      InteropUtils.GetSysVar("CMLSTYLE", ref str);
      m_sysVars.Add(new SysVarDataString("CMLSTYLE", false, "Stil fr neu erzeugte Multilinien", str));

      // CPLOTSTYLE fehlt
      InteropUtils.GetSysVar("CPLOTSTYLE", ref str);
      m_sysVars.Add(new SysVarDataString("CPLOTSTYLE", true, "Steuert den aktuellen Plotstil fr neue Objekte", str));

      // CTAB fehlt
      InteropUtils.GetSysVar("CTAB", ref str);
      m_sysVars.Add(new SysVarDataString("CTAB", false, "Gibt den Namen der aktuellen Registerkarte in der Zeichnung zurck (Modell oder Layout)", str));

      // CTABLESTYLE fehlt
      InteropUtils.GetSysVar("CTABLESTYLE", ref str);
      m_sysVars.Add(new SysVarDataString("CTABLESTYLE", false, "Stellt den Namen des aktuellen Tabellenstils ein", str));

      // CVPORT
      InteropUtils.GetSysVar("CVPORT", ref i);
      m_sysVars.Add(new SysVarDataInt("CVPORT", false, "Legt die Kennnummer des aktuellen Ansichtsfensters fest. Sie knnen diesen Wert ndern und damit das aktuelle Ansichtsfenster wechseln", i));

      // DBCSTATE
      InteropUtils.GetSysVar("DBCSTATE", ref i);
      m_sysVars.Add(new SysVarDataBool("DBCSTATE", true, "Speichert den Status des dbConnect-Managers (aktiv oder nicht aktiv)", Convert.ToBoolean(i)));
      
      m_sysVars.Add(new SysVarDataInt("DIMADEC", false, "Steuert die Anzahl der angezeigten Genauigkeitsstellen bei Winkelbemaungen (-1 = DIMDEC oder 0-8)", m_db.Dimadec));
      m_sysVars.Add(new SysVarDataBool("DIMALT", false, "Steuert die Anzeige von Bemaungen mit Alternativeinheiten (Ein oder Aus)", m_db.Dimalt));
      m_sysVars.Add(new SysVarDataInt("DIMALTD", false, "Steuert die Anzahl der Dezimalstellen fr Alternativeinheiten", m_db.Dimaltd));
      m_sysVars.Add(new SysVarDataDouble("DIMALTF", false, "Multiplikator fr Alternativeinheiten (Vorgabe 25.4 (Zoll->Millimeter))", m_db.Dimaltf));
      m_sysVars.Add(new SysVarDataDouble("DIMALTRND", false, "Rundet die Alternativeinheit fr Bemaungen", m_db.Dimaltrnd));
      m_sysVars.Add(new SysVarDataInt("DIMALTTD", false, "Setzt die Anzahl der Dezimalstellen fr die Toleranzwerte einer Bemaung mit Alternativeinheiten", m_db.Dimalttd));
      m_sysVars.Add(new SysVarDataInt("DIMALTTZ", false, "Schaltet die Unterdrckung von Nullen in Toleranzangaben um", m_db.Dimalttz));
      m_sysVars.Add(new SysVarDataInt("DIMALTU", false, "Setzt das Einheitenformat fr Alternativeinheiten bezglich aller Bemaungsstilarten auer Winkel", m_db.Dimaltu));
      m_sysVars.Add(new SysVarDataInt("DIMALTZ", false, "Aktiviert bzw. deaktiviert das Unterdrcken von Nullen fr Alternativeinheiten-Bemaungswerte", m_db.Dimaltz));
      m_sysVars.Add(new SysVarDataString("DIMAPOST", false, m_db.Dimapost));
      m_sysVars.Add(new SysVarDataBool("DIMASO", false, "Steuert die Assoziativitt von Bemaungsobjekten. Veraltet. Siehe DIMASSOC", m_db.Dimaso));
      m_sysVars.Add(new SysVarDataInt("DIMASSOC", false, "Steuert die Assoziativitt von Bemaungsobjekten (0 = aufgelst, 1 = nicht-assoziativ, 2 = assoziativ)", m_db.DimAssoc));
      m_sysVars.Add(new SysVarDataDouble("DIMASZ", false, "Steuert die Gre der Pfeilspitzen von Ma- und Fhrungslinien", m_db.Dimasz));
      m_sysVars.Add(new SysVarDataInt("DIMATFIT", false, "Legt fest, auf welche Weise der Matext und die Pfeile angeordnet werden sollen, wenn nicht ausreichend Platz fr alle Elemente zwischen den Hilfslinien verfgbar ist", m_db.Dimatfit));
      m_sysVars.Add(new SysVarDataInt("DIMAUNIT", false, "Einheitenformat fr Winkelbemaungen (0 = DezGrad, 1 = GradMinutenSek, 2 = Neugrad, 3 = Bogenma", m_db.Dimaunit));
      m_sysVars.Add(new SysVarDataInt("DIMAZIN", false, "Unterdrckt Nullen bei Winkelbemaungen", m_db.Dimazin));

      InteropUtils.GetSysVar("DIMBLK", ref str);
      m_sysVars.Add(new SysVarDataString("DIMBLK", false, "Bestimmt den Pfeilspitzenblock, der an den Enden der Malinien oder Fhrungslinien angezeigt wird (\".\" = Vorgabe)", str));
      
      InteropUtils.GetSysVar("DIMBLK1", ref str);
      m_sysVars.Add(new SysVarDataString("DIMBLK1", false, "Setzt die Pfeilspitze fr das erste Ende der Malinie, wenn DIMSAH aktiviert ist", str));

      InteropUtils.GetSysVar("DIMBLK2", ref str);
      m_sysVars.Add(new SysVarDataString("DIMBLK2", false, "Setzt die Pfeilspitze fr das zweite Ende der Malinie, wenn DIMSAH aktiviert ist", str));

      m_sysVars.Add(new SysVarDataDouble("DIMCEN", false, "Steuert das Zeichnen der Zentrumspunkte von Kreisen oder Bogen sowie der Mittellinien", m_db.Dimcen));
      m_sysVars.Add(new SysVarDataString("DIMCLRD", false, "Weist Malinien, Pfeilspitzen und Fhrungslinien Farben zu. Steuert auch die Farben von Fhrungslinien", m_db.Dimclrd.ToString()));
      m_sysVars.Add(new SysVarDataString("DIMCLRE", false, "Weist Mahilfslinien Farben zu", m_db.Dimclre.ToString()));
      m_sysVars.Add(new SysVarDataString("DIMCLRT", false, "Weist Matexten Farben zu", m_db.Dimclrt.ToString()));
      m_sysVars.Add(new SysVarDataInt("DIMDEC", false, "Legt die Anzahl der fr die Primreinheiten einer Bemaung angezeigten Dezimalstellen fest", m_db.Dimdec));
      m_sysVars.Add(new SysVarDataDouble("DIMDLE", false, "Abstand, um den die Malinie ber die Hilfslinie hinaus verlngert wird, wenn Schrgstriche gezeichnet werden", m_db.Dimdle));
      m_sysVars.Add(new SysVarDataDouble("DIMDLI", false, "Steuert die Abstnde von Malinien in Basislinieneinheiten", m_db.Dimdli));
      m_sysVars.Add(new SysVarDataString("DIMDSEP", false, "Definiert ein einstelliges Dezimaltrennzeichen, das bei Bemaungen mit dezimalem Einheitenformat verwendet werden soll", m_db.Dimdsep.ToString()));
      m_sysVars.Add(new SysVarDataDouble("DIMEXE", false, "Gibt an, wie weit eine Hilfslinie ber die Malinie hinausreichen soll", m_db.Dimexe));
      m_sysVars.Add(new SysVarDataDouble("DIMEXO", false, "Bestimmt, in welchem Abstand von den Ursprungspunkten Hilfslinien gezeichnet werden", m_db.Dimexo));
      m_sysVars.Add(new SysVarDataInt("DIMFRAC", false, "Legt die Bruchschreibweise fest, wenn DIMLUNIT auf 4 (Architectural) oder 5 (Bruch) gesetzt ist", m_db.Dimfrac));
      m_sysVars.Add(new SysVarDataDouble("DIMGAP", false, "Abstand um den Matext, der verwendet wird, wenn die Malinie durch den Matext unterbrochen wird", m_db.Dimgap));
      m_sysVars.Add(new SysVarDataInt("DIMJUST", false, "Steuert die horizontale Positionierung von Matext", m_db.Dimjust));
      
      InteropUtils.GetSysVar("DIMLDRBLK", ref str);
      m_sysVars.Add(new SysVarDataString("DIMLDRBLK", false, "Legt den Typ der Pfeilspitze fr Fhrungen fest", str));

      m_sysVars.Add(new SysVarDataDouble("DIMLFAC", false, "Setzt einen Skalierfaktor fr lineare Bemaungen", m_db.Dimlfac));
      m_sysVars.Add(new SysVarDataBool("DIMLIM", false, "Magrenzen werden als Vorgabetext erzeugt. Wenn Sie DIMLIM aktivieren, wird DIMTOL deaktiviert", m_db.Dimlim));
      m_sysVars.Add(new SysVarDataInt("DIMLUNIT", false, "Setzt Einheiten fr alle Bemaungstypen, auer der Winkelbemaung", m_db.Dimlunit));
      m_sysVars.Add(new SysVarDataString("DIMLWD", false, "Weist Malinien eine Linienstrke zu (-1 = VONLAYER, -2 = VONBLOCK, -3 = VORGABE oder Angaben in 1/100mm)", m_db.Dimlwd.ToString()));
      m_sysVars.Add(new SysVarDataString("DIMLWE", false, "Weist Hilfslinien eine Linienstrke zu (-1 = VONLAYER, -2 = VONBLOCK, -3 = VORGABE oder Angaben in 1/100mm)", m_db.Dimlwe.ToString()));
      m_sysVars.Add(new SysVarDataString("DIMPOST", false, "Legt fr die Bemaung ein Textprfix und/oder -suffix fest", m_db.Dimpost));
      m_sysVars.Add(new SysVarDataDouble("DIMRND", false, "Rundet alle Bemaungsabstnde auf den angegebenen Wert", m_db.Dimrnd));
      m_sysVars.Add(new SysVarDataBool("DIMSAH", false, "Steuert die Anzeige von Pfeilspitzenblcken fr Malinien (DIMBLK oder DIMBLK1/2 anwenden)", m_db.Dimsah));
      m_sysVars.Add(new SysVarDataDouble("DIMSCALE", false, "Globalen Skalierfaktor fr Bemaungsvariablen, die Gren, Abstnde oder Versatzwerte festlegen", m_db.Dimscale));
      m_sysVars.Add(new SysVarDataBool("DIMSD1", false, "Steuert die Unterdrckung der ersten Malinie", m_db.Dimsd1));
      m_sysVars.Add(new SysVarDataBool("DIMSD2", false, "Steuert die Unterdrckung der zweiten Malinie", m_db.Dimsd2));
      m_sysVars.Add(new SysVarDataBool("DIMSE1", false, "Unterdrckt die Anzeige der ersten Hilfslinie", m_db.Dimse1));
      m_sysVars.Add(new SysVarDataBool("DIMSE2", false, "Unterdrckt die Anzeige der zweiten Hilfslinie", m_db.Dimse2));
      m_sysVars.Add(new SysVarDataBool("DIMSOXD", false, "Unterdrckt das Zeichnen von Malinien auerhalb der Hilfslinien", m_db.Dimsoxd));

      InteropUtils.GetSysVar("DIMSTYLE", ref str);
      m_sysVars.Add(new SysVarDataString("DimstyleData", true, str));

      m_sysVars.Add(new SysVarDataInt("DIMTAD", false, "Steuert die vertikale Position von Text in Bezug auf eine Malinie", m_db.Dimtad));
      m_sysVars.Add(new SysVarDataInt("DIMTDEC", false, "Legt die Anzahl der anzuzeigenden Dezimalstellen fr Toleranzwertebemaung mit Primreinheiten fest", m_db.Dimtdec));
      m_sysVars.Add(new SysVarDataDouble("DIMTFAC", false, "Skalierfaktor fr die Texthhe von Brchen und Toleranzwerten relativ zur mit DIMTXT festgelegten Hhe des Matextes", m_db.Dimtfac));
      m_sysVars.Add(new SysVarDataBool("DIMTIH", false, "Steuert die Position von Matext innerhalb der Hilfslinien fr alle Bemaungstypen auer Koordinatenbemaungen", m_db.Dimtih));
      m_sysVars.Add(new SysVarDataBool("DIMTIX", false, "Zeichnet Text zwischen den Hilfslinien (EIN = Matext auch dann zwischen den Hilfslinien, wenn er normalerweise auerhalb platziert wrde)", m_db.Dimtix));
      m_sysVars.Add(new SysVarDataDouble("DIMTM", false, "Wenn DIMTOL oder DIMLIM aktiviert ist, legt diese Variable die minimale (untere) Toleranzgrenze fr Matext fest", m_db.Dimtm));
      m_sysVars.Add(new SysVarDataInt("DIMTMOVE", false, "Setzt die Regeln fr die Verschiebung von Matext (Fhrung usw.)", m_db.Dimtmove));
      m_sysVars.Add(new SysVarDataBool("DIMTOFL", false, "Steuert, ob zwischen den Hilfslinien eine Malinie gezeichnet werden soll, wenn der Text auerhalb der Hilfslinien platziert wird", m_db.Dimtofl));
      m_sysVars.Add(new SysVarDataBool("DIMTOH", false, "Steuert die Position des Matexts auerhalb der Hilfslinien", m_db.Dimtoh));
      m_sysVars.Add(new SysVarDataBool("DIMTOL", false, "Fgt dem Matext Toleranzen hinzu. Wenn Sie DIMTOL aktivieren, wird DIMLIM deaktiviert", m_db.Dimtol));
      m_sysVars.Add(new SysVarDataInt("DIMTOLJ", false, "Setzt fr Toleranzwerte die vertikale Ausrichtung relativ zum nominalen Matext", m_db.Dimtolj));
      m_sysVars.Add(new SysVarDataDouble("DIMTP", false, "Wenn DIMTOL oder DIMLIM aktiviert ist, legt diese Variable die maximale (obere) Toleranzgrenze fr Matext fest", m_db.Dimtp));
      m_sysVars.Add(new SysVarDataDouble("DIMTSZ", false, "Gre der Schrgstriche, die statt Pfeilspitzen fr Linear-, Radius- und Durchmesserbemaungen gezeichnet werden", m_db.Dimtsz));
      m_sysVars.Add(new SysVarDataDouble("DIMTVP", false, "Steuert die vertikale Position von Matext oberhalb oder unterhalb der Malinie", m_db.Dimtvp));
      m_sysVars.Add(new SysVarDataString("DIMTXSTY", false, "Bestimmt den Textstil der Bemaung", Utils.SymbolIdToName(m_db.Dimtxsty)));
      m_sysVars.Add(new SysVarDataDouble("DIMTXT", false, "Legt die Hhe des Matextes fest, auer wenn der aktuelle Textstil eine feste Hhe besitzt", m_db.Dimtxt));
      m_sysVars.Add(new SysVarDataInt("DIMTZIN", false, "Steuert die Unterdrckung von Nullen in Toleranzwerten", m_db.Dimtzin));
      m_sysVars.Add(new SysVarDataBool("DIMUPT", false, "Steuert die Optionen fr vom Benutzer positionierten Text", m_db.Dimupt));
      m_sysVars.Add(new SysVarDataInt("DIMZIN", false, "Steuert die Unterdrckung von Nullen im Primreinheitenwert", m_db.Dimzin));
      m_sysVars.Add(new SysVarDataBool("DISPSILH", false, "Steuert im Drahtmodellmodus die Anzeige von Umrisskurven und Netzen von Volumenkrpern", m_db.DispSilh));
      m_sysVars.Add(new SysVarDataInt("DRAWORDERCTL", false, "Steuert die Zeichnungsreihenfolge-Funktionalitt", m_db.DrawOrderCtl));

      InteropUtils.GetSysVar("DWGCODEPAGE", ref str);
      m_sysVars.Add(new SysVarDataString("DWGCODEPAGE", true, "Speichert denselben Wert wie SYSCODEPAGE (aus Kompatibilittsgrnden)", str));

      m_sysVars.Add(new SysVarDataDouble("ELEVATION", false, "Speichert die aktuelle Erhebung relativ zum aktuellen BKS", m_db.Elevation));

      // EndCaps - was ist das???
      m_sysVars.Add(new SysVarDataString("EndCaps", false, m_db.EndCaps.ToString()));

      m_sysVars.Add(new SysVarDataPoint3d("EXTMAX", true, "Speichert den rechten oberen Punkt der Zeichnungsabmessungen", m_db.Extmax));
      m_sysVars.Add(new SysVarDataPoint3d("EXTMIN", true, "Speichert den linken unteren Punkt der Zeichnungsabmessungen", m_db.Extmin));
      m_sysVars.Add(new SysVarDataBool("EXTNAMES", false, "Legt die Parameter fr die Namen von benannten Objekten fest, die in Definitionstabellen gespeichert werden", m_db.AllowExtendedNames));
      m_sysVars.Add(new SysVarDataDouble("FACETRES", false, "Bewirkt eine Anpassung der Gltte von gerenderten und schattierten Objekten (0.01 - 10)", m_db.Facetres));
      m_sysVars.Add(new SysVarDataDouble("FILLETRAD", false, "Speichert den aktuellen Rundungsradius", m_db.Filletrad));
      m_sysVars.Add(new SysVarDataBool("FILLMODE", false, "Legt fest, ob Schraffuren, 2D-Solids und breite Polylinien ausgefllt werden", m_db.Fillmode));
      
      InteropUtils.GetSysVar("GRIDMODE", ref i);
      m_sysVars.Add(new SysVarDataBool("GRIDMODE", false, "Legt fest, ob das Raster aktiviert oder deaktiviert ist", Convert.ToBoolean(i)));
      
      InteropUtils.GetSysVar("GRIDUNIT", ref p2d);
      m_sysVars.Add(new SysVarDataPoint2d("GRIDUNIT", false, "Legt den Rasterabstand (X und Y) fr das aktuelle Ansichtsfenster fest", p2d));

      m_sysVars.Add(new SysVarDataInt("HALOGAP", false, "Gibt den Abstand an, der angezeigt wird, wenn ein Objekt durch ein anderes verdeckt wird", m_db.HaloGap));

      // Was ist HANDSEED?
      m_sysVars.Add(new SysVarDataString("Handseed", true, "HANDSEED is a counter which maintains the next available handle number", m_db.Handseed.ToString()));

      m_sysVars.Add(new SysVarDataInt("HIDETEXT", false, "Legt fest, ob Textobjekte verarbeitet werden knnen, whrend der Befehl VERDECKT aktiv ist", m_db.HideText));
      m_sysVars.Add(new SysVarDataString("HYPERLINKBASE", false, "Legt den Pfad fr alle relativen Hyperlinks in der Zeichnung fest", m_db.HyperlinkBase));
      m_sysVars.Add(new SysVarDataPoint3d("INSBASE", false, "Speichert den mit BASIS gesetzten Basispunkt, der in BKS-Koordinaten fr den aktuellen Bereich angegeben wird", m_db.Insbase));
      m_sysVars.Add(new SysVarDataString("INSUNITS", false, "Zeicheneinheitswert fr die automatische Skalierung von Blcken oder Bildern fest, die aus dem DesignCenter eingefgt werden", m_db.Insunits.ToString()));
      
      // Bei den nchsten beiden weicht die Schreibweise ab
      m_sysVars.Add(new SysVarDataInt("INTERSECTIONCOLOR", false, "Legt die Farbe von Flchenschnittpunkten fest (0 = VONBLOCK, 256 = VONLAYER, 257 = VONOBJEKT)", m_db.IntersectColor));
      m_sysVars.Add(new SysVarDataInt("INTERSECTIONDISPLAY", false, "Legt die Anzeige von Flchenschnittpunkten fest", m_db.IntersectDisplay));
      
      // Was ist das???
      m_sysVars.Add(new SysVarDataBool("IsEmr", true, "True if this drawing database was created by a version of AutoCAD meant for an educational institution instead of a standard AutoCAD version", m_db.IsEmr));

      m_sysVars.Add(new SysVarDataInt("ISOLINES", false, "Bestimmt auf Objekten die Anzahl der Konturlinien pro Oberflche", m_db.Isolines));
      
      // Wieder keine Ahnung, was das ist:
      m_sysVars.Add(new SysVarDataString("JoinStyle", false, "Returns the database's joinstyle header variable value, which may be kJnStylNone, kJnStylRound, kJnStylAngle, kJnStylFlat", m_db.JoinStyle.ToString()));

      m_sysVars.Add(new SysVarDataBool("LIMCHECK", false, "Steuert die Erzeugung von Objekten auerhalb der Rastergrenzen", m_db.Limcheck));
      m_sysVars.Add(new SysVarDataPoint2d("LIMMAX", false, "Speichert die oberen rechten Rastergrenzen des aktuellen Bereichs (in Weltkoordinaten)", m_db.Limmax));
      m_sysVars.Add(new SysVarDataPoint2d("LIMMIN", false, "Speichert die unteren linken Rastergrenzen des aktuellen Bereichs (in Weltkoordinaten)", m_db.Limmin));
      
      InteropUtils.GetSysVar("LOGFILENAME", ref str);
      m_sysVars.Add(new SysVarDataString("LOGFILENAME", true, "Legt den Pfad und den Namen der Protokolldatei fr die aktuelle Zeichnung fest", str));

      m_sysVars.Add(new SysVarDataDouble("LTSCALE", false, "Legt den globalen Skalierfaktor fr Linientypen fest. Der Skalierfaktor darf nicht auf null gesetzt werden", m_db.Ltscale));
      m_sysVars.Add(new SysVarDataInt("LUNITS", false, "Bestimmt lineare Einheiten (1 Wissensch., 2 Dezimal, 3 Maschinenbau, 4 Architektur, 5 Bruch)", m_db.Lunits));
      m_sysVars.Add(new SysVarDataInt("LUPREC", false, "Bestimmt die Anzahl der angezeigten Dezimalstellen fr Einheiten", m_db.Luprec));

      // Abweichender Name
      m_sysVars.Add(new SysVarDataBool("LWDISPLAY", false, "Steuert, ob die Linienstrke angezeigt wird. Diese Einstellung wird mit den einzelnen Registerkarten in der Zeichnung gespeichert", m_db.LineWeightDisplay));
      
      m_sysVars.Add(new SysVarDataInt("MAXACTVP", false, "Definiert die maximale Anzahl der Ansichtsfenster, die zur gleichen Zeit in einem Layout aktiv sein knnen", m_db.Maxactvp));
      m_sysVars.Add(new SysVarDataString("MEASUREMENT", false, "Zeichnungseinheiten fr die aktuelle Zeichnung (Britisch oder Metrisch). Steuert, welche Schraffurmuster- und Linientypdateien verwendet werden", m_db.Measurement.ToString()));
      m_sysVars.Add(new SysVarDataBool("MIRRTEXT", false, "Steuert die Anzeige von Text mit dem Befehl SPIEGELN", m_db.Mirrtext));
      
      // Es fehlt MSOLESCALE
      InteropUtils.GetSysVar("MSOLESCALE", ref d);
      m_sysVars.Add(new SysVarDataDouble("MSOLESCALE", false, "Steuert die Gre eines OLE-Objekts mit Text, das in den Modellbereich eingefgt wird", d));
      
      m_sysVars.Add(new SysVarDataInt("OBSCUREDCOLOR", false, "Legt die Farbe von verdeckten Linien fest (0 = VonBlock, 256 = VonLayer, 257 = VonObjekt)", m_db.ObscuredColor));
      
      // Namensabweichung
      m_sysVars.Add(new SysVarDataInt("OBSCUREDLTYPE", false, "Legt den Linientyp von verdeckten Linien fest. Im Gegensatz zu regulren Linientypen erhalten sich verborgene Linientypen unabhngig von Zoom-Ebenen", m_db.ObscuredLineType));
      
      InteropUtils.GetSysVar("OLEFRAME", ref i);
      m_sysVars.Add(new SysVarDataInt("OLEFRAME", false, "Steuert, ob ein Rahmen in allen OLE-Objekten in der Zeichnung angezeigt und geplottet wird", i));

      m_sysVars.Add(new SysVarDataBool("OLESTARTUP", false, "Steuert, ob die Quellanwendungen von eingebetteten OLE-Objekten beim Plotten geladen werden", m_db.OleStartUp));
      m_sysVars.Add(new SysVarDataBool("ORTHOMODE", false, "Beschrnkt Cursorbewegungen auf die Lotrechte", m_db.Orthomode));
      m_sysVars.Add(new SysVarDataInt("PDMODE", false, "Steuert, wie Punktobjekte angezeigt werden", m_db.Pdmode));
      m_sysVars.Add(new SysVarDataDouble("PDSIZE", false, "Legt die Anzeigegre von Punktobjekten fest", m_db.Pdsize));
      
      m_sysVars.Add(new SysVarDataBool("PELLIPSE", false, "Steuert den mit ELLIPSE erstellten Ellipsentyp (0 = Ellipse, 1 = Polylinie)", m_db.PlineEllipse));

      // Die fehlen alle in der Hilfe, und es gibt sie auch nicht als Sysvars
      m_sysVars.Add(new SysVarDataDouble("Pelevation", true, m_db.Pelevation));
      m_sysVars.Add(new SysVarDataPoint3d("Pextmax", true, m_db.Pextmax));
      m_sysVars.Add(new SysVarDataPoint3d("Pextmin", true, m_db.Pextmin));
      m_sysVars.Add(new SysVarDataPoint3d("Pinsbase", true, m_db.Pinsbase));
      m_sysVars.Add(new SysVarDataBool("Plimcheck", true, m_db.Plimcheck));
      m_sysVars.Add(new SysVarDataPoint2d("Plimmax", true, m_db.Plimmax));
      m_sysVars.Add(new SysVarDataPoint2d("Plimmin", true, m_db.Plimmin));

      m_sysVars.Add(new SysVarDataBool("PLINEGEN", false, "Bestimmt, wie Linientypmuster in der Umgebung der Kontrollpunkte einer zweidimensionalen Polylinie erzeugt werden", m_db.Plinegen));

      InteropUtils.GetSysVar("PLINETYPE", ref i);
      m_sysVars.Add(new SysVarDataInt("PLINETYPE", false, "Bestimmt, ob AutoCAD optimierte zweidimensionale Polylinien verwendet", i));

      m_sysVars.Add(new SysVarDataDouble("PLINEWID", false, "Speichert die Vorgabebreite fr Polylinien", m_db.Plinewid));

      // Es fehlt PROJECTNAME
      InteropUtils.GetSysVar("PROJECTNAME", ref str);
      m_sysVars.Add(new SysVarDataString("PROJECTNAME", false, "Weist der aktuellen Zeichnung einen Projektnamen zu", str));

      // Es fehlt PROXYGRAPHICS
      InteropUtils.GetSysVar("PROXYGRAPHICS", ref i);
      m_sysVars.Add(new SysVarDataBool("PROXYGRAPHICS", false, "Bestimmt, ob grafische Darstellungen von Proxy-Objekten in die Zeichnung aufgenommen werden sollen", Convert.ToBoolean(i)));

      m_sysVars.Add(new SysVarDataBool("PSLTSCALE", false, "Steuert die Skalierung von Linientypen im Papierbereich", m_db.Psltscale));

      // Namensabweichung
      m_sysVars.Add(new SysVarDataBool("PSTYLEMODE", true, "Zeigt an, ob die aktuelle Zeichnung den Modus Farbabhngige Plotstile oder Benannte Plotstile aufweist", m_db.PlotStyleMode));

      // Es fehlt PSVPSCALE
      InteropUtils.GetSysVar("PSVPSCALE", ref d);
      m_sysVars.Add(new SysVarDataDouble("PSVPSCALE", false, "Legt den Anzeige-Skalierungsfaktor fr alle neu erstellten Ansichtsfenster fest", d));

      InteropUtils.GetSysVar("PUCSBASE", ref str);
      m_sysVars.Add(new SysVarDataString("PUCSBASE", false, "Speichert den Namen des BKS, mit dem der Ursprung und die Ausrichtung fr die orthogonalen BKS-Einstellungen festgelegt werden (nur Papierbereich)", str));
      
      // Gibt's nicht als Systemvariable
      //InteropUtils.GetSysVar("PUCSNAME", ref str);
      //m_sysVars.Add(new SysVarDataString("PUCSNAME", str));
      
      // Gibt's nicht als Systemvariable
      m_sysVars.Add(new SysVarDataPoint3d("Pucsorg", true, m_db.Pucsorg));

      // Gibt's nicht als Systemvariable
      m_sysVars.Add(new SysVarDataString("PucsOrthographic", true, m_db.PucsOrthographic.ToString()));

      // Gibt's nicht als Systemvariable
      m_sysVars.Add(new SysVarDataVector3d("Pucsxdir", true, m_db.Pucsxdir));
 
      // Gibt's nicht als Systemvariable
      m_sysVars.Add(new SysVarDataVector3d("Pucsydir", true, m_db.Pucsydir));

      m_sysVars.Add(new SysVarDataBool("QTEXTMODE", false, "Steuert, wie Text angezeigt wird (0 = Text, 1 = Rahmen)", m_db.Qtextmode));
      m_sysVars.Add(new SysVarDataBool("REGENMODE", false, "Steuert die automatische Regenerierung von Zeichnungen", m_db.Regenmode));
      m_sysVars.Add(new SysVarDataInt("SHADEDGE", false, "Steuert das Schattieren von Kanten bei einem Rendering", m_db.Shadedge));
      m_sysVars.Add(new SysVarDataInt("SHADEDIF", false, "Bestimmt das Verhltnis des diffusen Reflexionslichts zum Umgebungslicht", m_db.Shadedif));
      m_sysVars.Add(new SysVarDataDouble("SKETCHINC", false, "Bestimmt die Skizziergenauigkeit fr den Befehl SKIZZE", m_db.Sketchinc));
      m_sysVars.Add(new SysVarDataBool("SKPOLY", false, "Legt fest, ob mit dem Befehl SKIZZE Linien oder Polylinien erstellt werden", m_db.Skpoly));

      // Es fehlt SNAPANG
      InteropUtils.GetSysVar("SNAPANG", ref d);
      m_sysVars.Add(new SysVarDataDouble("SNAPANG", false, "Setzt Fang- und Rasterdrehwinkel fr das aktuelle Ansichtsfenster. Der angegebene Winkel ist relativ zum aktuellen BKS", d));

      // Es fehlt SNAPBASE
      InteropUtils.GetSysVar("SNAPBASE", ref p2d);
      m_sysVars.Add(new SysVarDataPoint2d("SNAPBASE", false, "Bestimmt unter Bezug auf das aktuelle BKS den Fang- und Rasterursprungspunkt fr das aktuelle Ansichtsfenster", p2d));
      
      // Es fehlt SNAPISOPAIR
      InteropUtils.GetSysVar("SNAPISOPAIR", ref i);
      m_sysVars.Add(new SysVarDataInt("SNAPISOPAIR", false, "Steuert die isometrische Ebene fr das aktuelle Ansichtsfenster( 0 = Links, 1 = Oben, 2 = Rechts)", i));
      
      // Es fehlt SNAPMODE
      InteropUtils.GetSysVar("SNAPMODE", ref i);
      m_sysVars.Add(new SysVarDataBool("SNAPMODE", false, "Aktiviert und deaktiviert den Fangmodus", Convert.ToBoolean(i)));
      
      // Es fehlt SNAPSTYL
      InteropUtils.GetSysVar("SNAPSTYL", ref i);
      m_sysVars.Add(new SysVarDataBool("SNAPSTYL", false, "Setzt den Fangstil fr das aktuelle Ansichtsfenster", Convert.ToBoolean(i)));
      
      // Es fehlt SNAPTYPE
      InteropUtils.GetSysVar("SNAPTYPE", ref i);
      m_sysVars.Add(new SysVarDataInt("SNAPTYPE", false, "Setzt den Fangstil fr das aktuelle Ansichtsfenster", i));
      
      // Es fehlt SNAPUNIT
      InteropUtils.GetSysVar("SNAPUNIT", ref p2d);
      m_sysVars.Add(new SysVarDataPoint2d("SNAPUNIT", false, "Legt den Fangwert fr das aktuelle Ansichtsfenster fest", p2d));

      // SORTENTS fehlt in der Hilfe
      m_sysVars.Add(new SysVarDataInt("SORTENTS", false, m_db.SortEnts));

      m_sysVars.Add(new SysVarDataBool("SPLFRAME", false, "Steuert die Anzeige von Splines und kurvenangeglichenen Polylinien", m_db.Splframe));
      m_sysVars.Add(new SysVarDataInt("SPLINESEGS", false, "Bestimmt die Anzahl von Liniensegmenten fr die einzelnen kurvenangeglichenen Polylinien", m_db.Splinesegs));
      m_sysVars.Add(new SysVarDataInt("SPLINETYPE", false, "Legt den Typ der Kurve fest, die mit der Option Spline des Befehls PEDIT erstellt wird", m_db.Splinetype));
      m_sysVars.Add(new SysVarDataString("STYLESHEET", false, m_db.StyleSheet));
      m_sysVars.Add(new SysVarDataInt("SURFTAB1", false, "Setzt die Anzahl der Tabellationen, die fr die Befehle REGELOB und TABOB erstellt werden sollen", m_db.Surftab1));
      m_sysVars.Add(new SysVarDataInt("SURFTAB2", false, "Stellt die Netzdichte in N-Richtung fr die Befehle ROTOB und KANTOB ein", m_db.Surftab2));
      m_sysVars.Add(new SysVarDataInt("SURFTYPE", false, "Steuert den Typ der Oberflche, die mit der Option Glttung des Befehls PEDIT geglttet werden soll", m_db.Surftype));
      m_sysVars.Add(new SysVarDataInt("SURFU", false, "Bestimmt die Oberflchendichte in M-Richtung fr die Option Glttung im Befehl PEDIT", m_db.Surfu));
      m_sysVars.Add(new SysVarDataInt("SURFV", false, "Bestimmt die Oberflchendichte in N-Richtung fr die Option Glttung im Befehl PEDIT", m_db.Surfv));
      
      // TARGET fehlt
      InteropUtils.GetSysVar("TARGET", ref p3d);
      m_sysVars.Add(new SysVarDataPoint3d("TARGET", true, "Speichert die Position des Zielpunkts (in BKS-Koordinaten) fr das aktuelle Ansichtsfenster", p3d));
      
      m_sysVars.Add(new SysVarDataString("TDCREATE", true, "Speichert Zeit und Datum der Zeichnungserstellung (lokale Angaben)", m_db.Tdcreate.ToString()));
      //m_sysVars.Add(new SysVarDataString("Tdindwg", "Speichert die Gesamtbearbeitungszeit", m_db.Tdindwg.ToString()));        // TBD: throws exception!
      m_sysVars.Add(new SysVarDataString("TDUCREATE", true, "Speichert Datum und Zeit der Zeichnungserstellung (universelle Angaben)", m_db.Tducreate.ToString()));
      m_sysVars.Add(new SysVarDataString("TDUPDATE", true, "Speichert Zeit und Datum des letzten Aktualisierungs-/Speichervorgangs (lokale Angaben)", m_db.Tdupdate.ToString()));
      //m_sysVars.Add(new SysVarDataString("Tdusrtimer", "Speichert den Stand der Benutzer-Stoppuhr", m_db.Tdusrtimer.ToString()));  // TBD: throws exception!
      m_sysVars.Add(new SysVarDataString("TDUUPDATE", true, "Speichert Zeit und Datum des letzten Aktualisierungs-/Speichervorgangs (universelle Angaben)", m_db.Tduupdate.ToString()));
      m_sysVars.Add(new SysVarDataDouble("TEXTSIZE", false, "Vorgabehhe fr neue Textobjekte, die mit dem aktuellen Textstil gezeichnet werden", m_db.Textsize));


      // TEXTSTYLE fehlt
      InteropUtils.GetSysVar("TEXTSTYLE", ref str);
      m_sysVars.Add(new SysVarDataString("TEXTSTYLE", false, "Definiert den Namen des aktuellen Textstils", str));

      // THICKNESS fehlt
      InteropUtils.GetSysVar("THICKNESS", ref d);
      m_sysVars.Add(new SysVarDataDouble("THICKNESS", false, "Legt die aktuelle Hhe fr 3D-Objekte fest", d));

      m_sysVars.Add(new SysVarDataBool("TILEMODE", false, "Macht die Registerkarte Modell oder die letzte Layout-Registerkarte zur aktuellen Registerkarte", m_db.TileMode));
      m_sysVars.Add(new SysVarDataDouble("TRACEWID", false, "Stellt die Vorgabe-Breite fr den Befehl BAND ein", m_db.Tracewid));
      m_sysVars.Add(new SysVarDataInt("TREEDEPTH", false, "Legt die maximale Tiefe fest, d. h. wie oft sich der baumartig strukturierte Raumindex verzweigen darf", m_db.Treedepth));
      m_sysVars.Add(new SysVarDataInt("TSTACKALIGN", false, "Steuert die vertikale Ausrichtung von gestapeltem Text (0 = Unten, 1 = Zentriert, 2 = Oben)", m_db.TStackAlign));
      m_sysVars.Add(new SysVarDataInt("TSTACKSIZE", false, "Steuert den Prozentsatz der Hhe fr den gestapelten Text relativ zur aktuellen Hhe des ausgewhlten Textes", m_db.TstackSize));

      InteropUtils.GetSysVar("UCSBASE", ref str);
      m_sysVars.Add(new SysVarDataString("UCSBASE", false, "Speichert den Namen des BKS, der Ursprung und Ausrichtung fr die orthogonalen BKS-Einstellungen bezeichnet", str));

      // UCSFOLLOW fehlt
      InteropUtils.GetSysVar("UCSFOLLOW", ref i);
      m_sysVars.Add(new SysVarDataBool("UCSFOLLOW", false, "Erzeugt bei jedem Wechsel von einem BKS zu einem anderen eine Draufsicht", Convert.ToBoolean(i)));

      InteropUtils.GetSysVar("UCSNAME", ref str);
      m_sysVars.Add(new SysVarDataString("UCSNAME", true, "Speichert den Namen des aktuellen Koordinatensystems fr das aktuelle Ansichtsfenster im aktuellen Bereich", str));
      
      m_sysVars.Add(new SysVarDataPoint3d("UCSORG", true, "Speichert den Ursprungspunkt des aktuellen Koordinatensystems fr das aktuelle Ansichtsfenster im aktuellen Bereich", m_db.Ucsorg));
      m_sysVars.Add(new SysVarDataString("UCSORTHO", false, "Bestimmt, ob die zugehrige Einstellung fr das orthogonale BKS automatisch wiederhergestellt werden soll, wenn Sie eine orthogonale Ansicht wiederherstellen", m_db.UcsOrthographic.ToString()));
      
      // UCSVP fehlt
      InteropUtils.GetSysVar("UCSVP", ref i);
      m_sysVars.Add(new SysVarDataInt("UCSVP", false, "Bestimmt, ob das BKS in den Ansichtsfenstern unverndert bleibt oder nderungen am BKS des aktuellen Ansichtsfensters bernommen werden", i));

      m_sysVars.Add(new SysVarDataVector3d("UCSXDIR", true, "Speichert die X-Richtung des aktuellen BKS im aktuellen Ansichtsfenster des aktuellen Bereichs", m_db.Ucsxdir));
      m_sysVars.Add(new SysVarDataVector3d("UCSYDIR", true, "Speichert die Y-Richtung des aktuellen BKS im aktuellen Ansichtsfenster des aktuellen Bereichs", m_db.Ucsydir));
      
      // UNDOCTL fehlt
      InteropUtils.GetSysVar("UNDOCTL", ref i);
      m_sysVars.Add(new SysVarDataInt("UNDOCTL", true, "Speichert einen Bitcode, der den Status fr die Optionen Auto und Steuern des Befehls ZURCK anzeigt", i));

      m_sysVars.Add(new SysVarDataBool("UndoRecording", true, m_db.UndoRecording));

      m_sysVars.Add(new SysVarDataInt("UNITMODE", false, "Steuert das Anzeigeformat fr Einheiten", m_db.Unitmode));

      // UPDATETHUMBNAIL fehlt
      InteropUtils.GetSysVar("UPDATETHUMBNAIL", ref i);
      m_sysVars.Add(new SysVarDataInt("UPDATETHUMBNAIL", false, "Steuert die Aktualisierung von Miniaturvoransichten im Manager fr Planungsunterlagen", i));

      m_sysVars.Add(new SysVarDataInt("USERI1", false, "USERI1, USERI2, USERI3, USERI4 und USERI5 werden zum Speichern und Abrufen von Ganzzahlwerten verwendet", m_db.Useri1));
      m_sysVars.Add(new SysVarDataInt("USERI2", false, "USERI1, USERI2, USERI3, USERI4 und USERI5 werden zum Speichern und Abrufen von Ganzzahlwerten verwendet", m_db.Useri2));
      m_sysVars.Add(new SysVarDataInt("USERI3", false, "USERI1, USERI2, USERI3, USERI4 und USERI5 werden zum Speichern und Abrufen von Ganzzahlwerten verwendet", m_db.Useri3));
      m_sysVars.Add(new SysVarDataInt("USERI4", false, "USERI1, USERI2, USERI3, USERI4 und USERI5 werden zum Speichern und Abrufen von Ganzzahlwerten verwendet", m_db.Useri4));
      m_sysVars.Add(new SysVarDataInt("USERI5", false, "USERI1, USERI2, USERI3, USERI4 und USERI5 werden zum Speichern und Abrufen von Ganzzahlwerten verwendet", m_db.Useri5));
      m_sysVars.Add(new SysVarDataDouble("USERR1", false, "USERR1, USERR2, USERR3, USERR4 und USERR5 werden zum Speichern und Abrufen von reellen Zahlen verwendet", m_db.Userr1));
      m_sysVars.Add(new SysVarDataDouble("USERR2", false, "USERR1, USERR2, USERR3, USERR4 und USERR5 werden zum Speichern und Abrufen von reellen Zahlen verwendet", m_db.Userr2));
      m_sysVars.Add(new SysVarDataDouble("USERR3", false, "USERR1, USERR2, USERR3, USERR4 und USERR5 werden zum Speichern und Abrufen von reellen Zahlen verwendet", m_db.Userr3));
      m_sysVars.Add(new SysVarDataDouble("USERR4", false, "USERR1, USERR2, USERR3, USERR4 und USERR5 werden zum Speichern und Abrufen von reellen Zahlen verwendet", m_db.Userr4));
      m_sysVars.Add(new SysVarDataDouble("USERR5", false, "USERR1, USERR2, USERR3, USERR4 und USERR5 werden zum Speichern und Abrufen von reellen Zahlen verwendet", m_db.Userr5));
      
      // USERS1-5 fehlen
      InteropUtils.GetSysVar("USERS1", ref str);
      m_sysVars.Add(new SysVarDataString("USERS1", false, "USERS1, USERS2, USERS3, USERS4 und USERS5 werden zum Speichern und Abrufen von Textzeichenfolgen verwendet", str));
      InteropUtils.GetSysVar("USERS2", ref str);
      m_sysVars.Add(new SysVarDataString("USERS2", false, "USERS1, USERS2, USERS3, USERS4 und USERS5 werden zum Speichern und Abrufen von Textzeichenfolgen verwendet", str));
      InteropUtils.GetSysVar("USERS3", ref str);
      m_sysVars.Add(new SysVarDataString("USERS3", false, "USERS1, USERS2, USERS3, USERS4 und USERS5 werden zum Speichern und Abrufen von Textzeichenfolgen verwendet", str));
      InteropUtils.GetSysVar("USERS4", ref str);
      m_sysVars.Add(new SysVarDataString("USERS4", false, "USERS1, USERS2, USERS3, USERS4 und USERS5 werden zum Speichern und Abrufen von Textzeichenfolgen verwendet", str));
      InteropUtils.GetSysVar("USERS5", ref str);
      m_sysVars.Add(new SysVarDataString("USERS5", false, "USERS1, USERS2, USERS3, USERS4 und USERS5 werden zum Speichern und Abrufen von Textzeichenfolgen verwendet", str));

      // Fehlt in der Hilfe
      m_sysVars.Add(new SysVarDataBool("Usrtimer", true, m_db.Usrtimer));

      // VIEWCTR fehlt
      InteropUtils.GetSysVar("VIEWCTR", ref p3d);
      m_sysVars.Add(new SysVarDataPoint3d("VIEWCTR", true, "Speichert den Ansichtsmittelpunkt des aktuellen Ansichtsfensters. Wird als BKS-Koordinate ausgedrckt", p3d));

      // VIEWDIR fehlt
      InteropUtils.GetSysVar("VIEWDIR", ref v3d);
      m_sysVars.Add(new SysVarDataVector3d("VIEWDIR", true, "Speichert die Ansichtsrichtung fr das aktuelle Ansichtsfenster. Wird in BKS-Koordinaten ausgedrckt", v3d));

      // VIEWMODE fehlt
      InteropUtils.GetSysVar("VIEWMODE", ref i);
      m_sysVars.Add(new SysVarDataInt("VIEWMODE", true, "Speichert den Ansichtsmodus fr das aktuelle Ansichtsfenster als Bitcode", i));

      // VIEWSIZE fehlt
      InteropUtils.GetSysVar("VIEWSIZE", ref d);
      m_sysVars.Add(new SysVarDataDouble("VIEWSIZE", true, "Speichert die Hhe des aktuellen Ansichtsfensters in Zeichnungseinheiten", d));

      // VIEWTWIST fehlt
      InteropUtils.GetSysVar("VIEWTWIST", ref d);
      m_sysVars.Add(new SysVarDataDouble("VIEWTWIST", true, "Speichert den Ansichtsdrehwinkel fr das aktuelle Ansichtsfenster", d));

      // Nicht in der Hilfe
      m_sysVars.Add(new SysVarDataDouble("ViewportScaleDefault", true, m_db.ViewportScaleDefault));

      m_sysVars.Add(new SysVarDataBool("VISRETAIN", false, "Steuert Sichtbarkeit, Farbe, Linientyp, Linienstrke und Plotstile von XRef-unabhngigen Layern (falls PSTYLEPOLICY auf 0 gesetzt ist)", m_db.Visretain));

      // VSMAX fehlt
      InteropUtils.GetSysVar("VSMAX", ref p3d);
      m_sysVars.Add(new SysVarDataPoint3d("VSMAX", true, "Speichert die rechte obere Ecke des virtuellen Bildschirms im aktuellen Ansichtsfenster. Wird als BKS-Koordinate ausgedrckt", p3d));

      // VSMIN fehlt
      InteropUtils.GetSysVar("VSMIN", ref p3d);
      m_sysVars.Add(new SysVarDataPoint3d("VSMIN", true, "Speichert die linke untere Ecke des virtuellen Bildschirms im aktuellen Ansichtsfenster. Wird als BKS-Koordinate ausgedrckt", p3d));

      m_sysVars.Add(new SysVarDataBool("WORLDVIEW", false, "Legt fest, ob sich die Eingaben zu den Befehlen 3DORBIT, DANSICHT und APUNKT auf das WKS (Vorgabe) oder auf das aktuelle BKS beziehen", m_db.Worldview));
    }


		/// <summary>
		/// Die verwendeten Ressourcen bereinigen.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
      System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Form1));
      this.mainMenu1 = new System.Windows.Forms.MainMenu();
      this.m_menuFile = new System.Windows.Forms.MenuItem();
      this.m_menuFileLoad = new System.Windows.Forms.MenuItem();
      this.m_menuFileSave = new System.Windows.Forms.MenuItem();
      this.m_menuFileSep1 = new System.Windows.Forms.MenuItem();
      this.m_menuFileSaveXSD = new System.Windows.Forms.MenuItem();
      this.m_menuFileSep2 = new System.Windows.Forms.MenuItem();
      this.m_menuFileQuit = new System.Windows.Forms.MenuItem();
      this.m_menuEdit = new System.Windows.Forms.MenuItem();
      this.m_menuEditAdoptmarked = new System.Windows.Forms.MenuItem();
      this.m_menuEditAdoptall = new System.Windows.Forms.MenuItem();
      this.m_menuEditSep1 = new System.Windows.Forms.MenuItem();
      this.m_menuEditCopy = new System.Windows.Forms.MenuItem();
      this.m_menuEditPaste = new System.Windows.Forms.MenuItem();
      this.m_menuView = new System.Windows.Forms.MenuItem();
      this.m_menuViewDiffonly = new System.Windows.Forms.MenuItem();
      this.m_menuViewNodims = new System.Windows.Forms.MenuItem();
      this.m_menuHelp = new System.Windows.Forms.MenuItem();
      this.m_menuHelpContents = new System.Windows.Forms.MenuItem();
      this.m_menuHelpSep1 = new System.Windows.Forms.MenuItem();
      this.m_menuHelpAbout = new System.Windows.Forms.MenuItem();
      this.lv = new System.Windows.Forms.ListView();
      this.chVarname = new System.Windows.Forms.ColumnHeader();
      this.chLeft = new System.Windows.Forms.ColumnHeader();
      this.chRight = new System.Windows.Forms.ColumnHeader();
      this.chType = new System.Windows.Forms.ColumnHeader();
      this.chDescr = new System.Windows.Forms.ColumnHeader();
      this.statusBar = new System.Windows.Forms.StatusBar();
      this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
      this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
      this.SuspendLayout();
      // 
      // mainMenu1
      // 
      this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                              this.m_menuFile,
                                                                              this.m_menuEdit,
                                                                              this.m_menuView,
                                                                              this.m_menuHelp});
      // 
      // m_menuFile
      // 
      this.m_menuFile.Index = 0;
      this.m_menuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                               this.m_menuFileLoad,
                                                                               this.m_menuFileSave,
                                                                               this.m_menuFileSep1,
                                                                               this.m_menuFileSaveXSD,
                                                                               this.m_menuFileSep2,
                                                                               this.m_menuFileQuit});
      this.m_menuFile.Text = "Datei";
      // 
      // m_menuFileLoad
      // 
      this.m_menuFileLoad.Index = 0;
      this.m_menuFileLoad.Text = "ffnen";
      this.m_menuFileLoad.Click += new System.EventHandler(this.menuFileLoad_Click);
      // 
      // m_menuFileSave
      // 
      this.m_menuFileSave.Index = 1;
      this.m_menuFileSave.Text = "Speichern";
      this.m_menuFileSave.Click += new System.EventHandler(this.menuFileSave_Click);
      // 
      // m_menuFileSep1
      // 
      this.m_menuFileSep1.Index = 2;
      this.m_menuFileSep1.Text = "-";
      // 
      // m_menuFileSaveXSD
      // 
      this.m_menuFileSaveXSD.Index = 3;
      this.m_menuFileSaveXSD.Text = "XSD speichern";
      this.m_menuFileSaveXSD.Click += new System.EventHandler(this.menuFileSaveXSD_Click);
      // 
      // m_menuFileSep2
      // 
      this.m_menuFileSep2.Index = 4;
      this.m_menuFileSep2.Text = "-";
      // 
      // m_menuFileQuit
      // 
      this.m_menuFileQuit.Index = 5;
      this.m_menuFileQuit.Text = "Beenden";
      this.m_menuFileQuit.Click += new System.EventHandler(this.menuFileQuit_Click);
      // 
      // m_menuEdit
      // 
      this.m_menuEdit.Index = 1;
      this.m_menuEdit.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                               this.m_menuEditAdoptmarked,
                                                                               this.m_menuEditAdoptall,
                                                                               this.m_menuEditSep1,
                                                                               this.m_menuEditCopy,
                                                                               this.m_menuEditPaste});
      this.m_menuEdit.Text = "Bearbeiten";
      // 
      // m_menuEditAdoptmarked
      // 
      this.m_menuEditAdoptmarked.Index = 0;
      this.m_menuEditAdoptmarked.Text = "Markierte bernehmen";
      // 
      // m_menuEditAdoptall
      // 
      this.m_menuEditAdoptall.Index = 1;
      this.m_menuEditAdoptall.Text = "Alle bernehmen";
      // 
      // m_menuEditSep1
      // 
      this.m_menuEditSep1.Index = 2;
      this.m_menuEditSep1.Text = "-";
      // 
      // m_menuEditCopy
      // 
      this.m_menuEditCopy.Index = 3;
      this.m_menuEditCopy.Text = "Kopieren";
      // 
      // m_menuEditPaste
      // 
      this.m_menuEditPaste.Index = 4;
      this.m_menuEditPaste.Text = "Einfgen";
      // 
      // m_menuView
      // 
      this.m_menuView.Index = 2;
      this.m_menuView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                               this.m_menuViewDiffonly,
                                                                               this.m_menuViewNodims});
      this.m_menuView.Text = "Ansicht";
      // 
      // m_menuViewDiffonly
      // 
      this.m_menuViewDiffonly.Index = 0;
      this.m_menuViewDiffonly.Text = "Nur Unterschiede";
      // 
      // m_menuViewNodims
      // 
      this.m_menuViewNodims.Index = 1;
      this.m_menuViewNodims.Text = "Keine Bemaungsvariablen";
      this.m_menuViewNodims.Click += new System.EventHandler(this.menuViewNodims_Click);
      // 
      // m_menuHelp
      // 
      this.m_menuHelp.Index = 3;
      this.m_menuHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                               this.m_menuHelpContents,
                                                                               this.m_menuHelpSep1,
                                                                               this.m_menuHelpAbout});
      this.m_menuHelp.Text = "Hilfe";
      // 
      // m_menuHelpContents
      // 
      this.m_menuHelpContents.Index = 0;
      this.m_menuHelpContents.Text = "Inhalt";
      // 
      // m_menuHelpSep1
      // 
      this.m_menuHelpSep1.Index = 1;
      this.m_menuHelpSep1.Text = "-";
      // 
      // m_menuHelpAbout
      // 
      this.m_menuHelpAbout.Index = 2;
      this.m_menuHelpAbout.Text = "ber CompareSysVars";
      // 
      // lv
      // 
      this.lv.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
        | System.Windows.Forms.AnchorStyles.Left) 
        | System.Windows.Forms.AnchorStyles.Right);
      this.lv.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
                                                                         this.chVarname,
                                                                         this.chLeft,
                                                                         this.chRight,
                                                                         this.chType,
                                                                         this.chDescr});
      this.lv.GridLines = true;
      this.lv.Location = new System.Drawing.Point(8, 8);
      this.lv.Name = "lv";
      this.lv.Size = new System.Drawing.Size(664, 392);
      this.lv.TabIndex = 0;
      this.lv.View = System.Windows.Forms.View.Details;
      this.lv.SelectedIndexChanged += new System.EventHandler(this.lv_SelectedIndexChanged);
      // 
      // chVarname
      // 
      this.chVarname.Text = "Variablenname";
      this.chVarname.Width = 140;
      // 
      // chLeft
      // 
      this.chLeft.Text = "In Zeichnung";
      this.chLeft.Width = 150;
      // 
      // chRight
      // 
      this.chRight.Text = "Vergleich";
      this.chRight.Width = 150;
      // 
      // chType
      // 
      this.chType.Text = "Datentyp";
      this.chType.Width = 120;
      // 
      // chDescr
      // 
      this.chDescr.Text = "Beschreibung";
      this.chDescr.Width = 3000;
      // 
      // statusBar
      // 
      this.statusBar.Anchor = ((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
        | System.Windows.Forms.AnchorStyles.Right);
      this.statusBar.Dock = System.Windows.Forms.DockStyle.None;
      this.statusBar.Location = new System.Drawing.Point(0, 407);
      this.statusBar.Name = "statusBar";
      this.statusBar.Size = new System.Drawing.Size(1580, 24);
      this.statusBar.TabIndex = 1;
      // 
      // saveFileDialog1
      // 
      this.saveFileDialog1.FileName = "doc1";
      // 
      // Form1
      // 
      this.AutoScaleBaseSize = new System.Drawing.Size(6, 15);
      this.ClientSize = new System.Drawing.Size(680, 431);
      this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                  this.statusBar,
                                                                  this.lv});
      this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
      this.Menu = this.mainMenu1;
      this.Name = "Form1";
      this.Text = "CSysVars";
      this.ResumeLayout(false);

    }
		#endregion

		/// <summary>
		/// EventHandler fr Men Datei->Speichern
		/// </summary>
    private void menuFileSave_Click(object sender, System.EventArgs e) {
      saveFileDialog1.Filter = "XML-Dateien (*.xml)|*.xml";
      saveFileDialog1.InitialDirectory = ".";
      saveFileDialog1.FilterIndex = 0;

      if(saveFileDialog1.ShowDialog() == DialogResult.OK){
        ActiveForm.Refresh();
        XmlUtils.saveAsXml(saveFileDialog1.FileName, m_sysVars);
      }else{
        ActiveForm.Refresh();
      }
    }

		/// <summary>
		/// EventHandler fr Men Datei->ffnen
		/// </summary>
    private void menuFileLoad_Click(object sender, System.EventArgs e) {
      openFileDialog1.Filter = "XML-Dateien (*.xml)|*.xml";
      openFileDialog1.InitialDirectory = ".";
      openFileDialog1.FilterIndex = 0;

      if(openFileDialog1.ShowDialog() == DialogResult.OK){
        ActiveForm.Refresh();
        //string Result = XmlUtils.Validate(openFileDialog1.FileName, m_xsdValidationStream);
        string Result = string.Empty;
        if(Result == string.Empty){
          XmlDocument xmlDoc = XmlUtils.loadXml(openFileDialog1.FileName);
          importValues(xmlDoc);
          m_compareLoaded = true;
          displayData();
        }else{
          MessageBox.Show(Result);
        }
      }else{
        ActiveForm.Refresh();
      }

    }

		/// <summary>
		///Liest die Vergleichswerte aus einem geffneten XML-Document
		/// </summary>
    private void importValues(XmlDocument xmlDoc){
      for(int i = 0; i < m_sysVars.Count; i++){
        SysVarData item = (SysVarData)m_sysVars[i];
        string val = xmlDoc.SelectSingleNode("/SetOfVariables/" + item.Label).InnerText;
        switch(item.typeStr()){
        case "Ganzzahl": 
          ((SysVarDataInt)item).ValueR = Convert.ToInt32(val);
          break;
        case "Fliekommazahl": 
          ((SysVarDataDouble)item).ValueR = Convert.ToDouble(val);
          break;
        case "Zeichenkette": 
          ((SysVarDataString)item).ValueR = val;
          break;
        case "Boolean": 
          ((SysVarDataBool)item).ValueR = Convert.ToBoolean(val);
          break;
        case "Punkt3d":
          ((SysVarDataPoint3d)item).ValueR = Utils.StringToPoint3d(val);
          break;
        case "Punkt2d":
          ((SysVarDataPoint2d)item).ValueR = Utils.StringToPoint2d(val);
          break;
        case "Vektor3d":
          ((SysVarDataVector3d)item).ValueR = Utils.StringToVector3d(val);
          break;
        case "Vektor2d":
          ((SysVarDataVector2d)item).ValueR = Utils.StringToVector2d(val);
          break;
        }
      }

    }

		/// <summary>
		/// EventHandler fr Klicks auf einzelne Variablen
		/// </summary>
    private void lv_SelectedIndexChanged(object sender, System.EventArgs e) {
      if(lv.SelectedIndices.Count == 1){
        
        statusBar.Text = lv.Items[lv.SelectedIndices[0]].SubItems[4].Text;
      }else{
        statusBar.Text = lv.SelectedIndices.Count.ToString() + " Elemente gewhlt";
      }
    }

		/// <summary>
		/// EventHandler fr Men Datei->Beenden
		/// </summary>
    // kein Cleanup notwendig, es kann einfach so beendet werden
    // Da wir an die (statische) Variable 'dialog' aus 'App.cs' hier nicht
    // drankommen, gehen wir den kleinen Umweg ber FindForm, um die
    // zu schlieende Instanz des Formulars zu ermitteln
    private void menuFileQuit_Click(object sender, System.EventArgs e) {
      lv.FindForm().Close();
    }

		/// <summary>
		/// EventHandler fr Men Ansicht->Keine Bemaungsvariablen
		/// </summary>
    private void menuViewNodims_Click(object sender, System.EventArgs e) {
      m_showDimVars = !(m_menuViewNodims.Checked = m_showDimVars);
      displayData(); 
    }

		/// <summary>
		/// EventHandler fr Men Datei->XSD speichern
		/// </summary>
    private void menuFileSaveXSD_Click(object sender, System.EventArgs e) {
      saveFileDialog1.Filter = "XSD-Dateien (*.xsd)|*.xsd";
      saveFileDialog1.InitialDirectory = ".";
      saveFileDialog1.FilterIndex = 0;

      if(saveFileDialog1.ShowDialog() == DialogResult.OK){
        ActiveForm.Refresh();
        XmlDocument xsdDoc = new XmlDocument();
        xsdDoc.Load(m_xsdValidationStream);
        xsdDoc.Save(saveFileDialog1.FileName);
      }else{
        ActiveForm.Refresh();
      }
    
    }
  }
}
