using System;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;

namespace CSysVars{

  // Abstrakte Basisklasse fr die Datenstze
	public abstract class SysVarData{
    protected double m_epsilon = -1E12D;

    // Der Name der Systemvariablen
    private string m_label;

    // Kurze Beschreibung der Variablen
    private string m_descr;

    // Schreibschutz
    private bool m_ronly;

    // Ctor
		public SysVarData(string label, bool ronly, string descr){
      m_label = label;
      m_ronly = ronly;
      m_descr = descr;
		}

    // Ctor
		public SysVarData(string label, bool ronly){
      m_label = label;
      m_ronly = ronly;
      m_descr = "Keine Beschreibung vorhanden";
		}

    // Getter und Setter fr den Variablen-Namen
    public string Label{
      get { return m_label;  }
      set { m_label = value; }
   	}

    // Getter und Setter fr die Beschreibung
    public string Descr{
      get { return m_descr;  }
      set { m_descr = value; }
   	}

    // Rckgabe der Daten Links und Rechts als Zeichenkette
    public abstract string StrValueL();
    public abstract string StrValueR();

    // Testet, ob gleiche Werte vorliegen
    public abstract bool EqualValues();

    // Testet, ob eine DIM-Variable vorliegt
    public Boolean IsDimVar(){
      return (m_label.Substring(0, 3) == "DIM");
    }

    // Rckgabe des Datentyps als (deutscher) String
    public abstract string typeStr();
	}

  // Abgeleitete Klasse fr Datentyp Double
  public class SysVarDataDouble : SysVarData{
    // Linker Wert
	  protected double m_valL;
    // Rechter Wert
	  protected double m_valR;
	  // Ctor (nur linken Wert setzen) 
		public SysVarDataDouble(string label, bool ronly, string descr, double valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataDouble(string label, bool ronly, double valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public double ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public double ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      return m_valL.ToString(); 
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      return m_valR.ToString(); 
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      //return (Math.Abs(m_valL - m_valR) < m_epsilon);
      return this.StrValueL() == this.StrValueR();
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Fliekommazahl";
    }
	}

  // Abgeleitete Klasse fr Datentyp Int
  public class SysVarDataInt : SysVarData{
    // Linker Wert
	  protected int m_valL;
    // Rechter Wert
	  protected int m_valR;
	  // Ctor (nur linken Wert setzen) 
		public SysVarDataInt(string label, bool ronly, string descr, int valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataInt(string label, bool ronly, int valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public int ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public int ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      return m_valL.ToString(); 
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      return m_valR.ToString(); 
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Ganzzahl";
    }
  }

  // Abgeleitete Klasse fr Datentyp string
  public class SysVarDataString : SysVarData{
    // Linker Wert
	  protected string m_valL;
    // Rechter Wert
	  protected string m_valR;
 	  // Ctor (nur linken Wert setzen) 
  	public SysVarDataString(string label, bool ronly, string descr, string valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
  	public SysVarDataString(string label, bool ronly, string valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public String ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public String ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      return "\"" + m_valL + "\""; 
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      return "\"" + m_valR + "\""; 
    }
    // Rckgabe linker Wert als Zeichenkette
    public string UnquotedStrValueL(){
      return m_valL; 
    }
    // Rckgabe rechter Wert als Zeichenkette
    public string UnquotedStrValueR(){
      return m_valR; 
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Zeichenkette";
    }
	}

  // Abgeleitete Klasse fr Datentyp boolean
  public class SysVarDataBool : SysVarData{
    // Linker Wert
	  protected bool m_valL;
    // Rechter Wert
	  protected bool m_valR;
    // Ctor (nur linken Wert setzen)
		public SysVarDataBool(string label, bool ronly, string descr, Boolean valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataBool(string label, bool ronly, bool valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public bool ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public bool ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      return m_valL.ToString(); 
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      return m_valR.ToString(); 
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Boolean";
    }
	}

  // Nicht nutzbar - die ObjectId eines Layers in einer Zeichnung ist
  // eine andere als die des Layers in einer anderen Zeichnung!!!
  /*
  public class SysVarDataObjectId : SysVarData{
    // Linker Wert
	  protected ObjectId m_valL;
    // Rechter Wert
	  protected ObjectId m_valR;
    // Ctor (nur linken Wert setzen)
		public SysVarDataObjectId(string label, string descr, ObjectId valL) : base(label, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataObjectId(string label, ObjectId valL) : base(label){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public ObjectId ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public ObjectId ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      return m_valL.ToString(); 
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      return m_valR.ToString(); 
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "ObjectId";
    }
	}
  */

  // Abgeleitete Klasse fr Datentyp Point3d
  public class SysVarDataPoint3d : SysVarData{
    // Linker Wert
	  protected Point3d m_valL;
    // Rechter Wert
	  protected Point3d m_valR;
    // Ctor (nur linken Wert setzen)
		public SysVarDataPoint3d(string label, bool ronly, string descr, Point3d valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataPoint3d(string label, bool ronly, Point3d valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public Point3d ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public Point3d ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      //return m_valL.ToString(); 
      return ("(" + m_valL.X.ToString() + " " + m_valL.Y.ToString() + " " + m_valL.Z.ToString() + ")");
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      //return m_valR.ToString(); 
      return ("(" + m_valR.X.ToString() + " " + m_valR.Y.ToString() + " " + m_valR.Z.ToString() + ")");
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Punkt3d";
    }
}

  // Abgeleitete Klasse fr Datentyp Point2d
  public class SysVarDataPoint2d : SysVarData{
    // Linker Wert
	  protected Point2d m_valL;
    // Rechter Wert
	  protected Point2d m_valR;
    // Ctor (nur linken Wert setzen)
		public SysVarDataPoint2d(string label, bool ronly, string descr, Point2d valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataPoint2d(string label, bool ronly, Point2d valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public Point2d ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public Point2d ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      //return m_valL.ToString(); 
      return ("(" + m_valL.X.ToString() + " " + m_valL.Y.ToString() + ")");
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      //return m_valR.ToString(); 
      return ("(" + m_valR.X.ToString() + " " + m_valR.Y.ToString() + ")");
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Punkt2d";
    }
	}

  // Abgeleitete Klasse fr Datentyp Vector3d
  public class SysVarDataVector3d : SysVarData{
    // Linker Wert
	  protected Vector3d m_valL;
    // Rechter Wert
	  protected Vector3d m_valR;
    // Ctor (nur linken Wert setzen)
		public SysVarDataVector3d(string label, bool ronly, string descr, Vector3d valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataVector3d(string label, bool ronly, Vector3d valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public Vector3d ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public Vector3d ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      //return m_valL.ToString(); 
      return ("(" + m_valL.X.ToString() + " " + m_valL.Y.ToString() + " " + m_valL.Z.ToString() + ")");
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      //return m_valR.ToString(); 
      return ("(" + m_valR.X.ToString() + " " + m_valR.Y.ToString() + " " + m_valR.Z.ToString() + ")");
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Vektor3d";
    }
	}

  // Abgeleitete Klasse fr Datentyp Vector2d
  public class SysVarDataVector2d : SysVarData{
	  // Linker Wert
	  protected Vector2d m_valL;
    // Rechter Wert
    protected Vector2d m_valR;
    // Ctor (nur linken Wert setzen)
		public SysVarDataVector2d(string label, bool ronly, string descr, Vector2d valL) : base(label, ronly, descr){
		  m_valL = valL;
		}
	  // Ctor nur linken Wert setzen, keine Beschreibung 
		public SysVarDataVector2d(string label, bool ronly, Vector2d valL) : base(label, ronly){
		  m_valL = valL;
		}
    // Getter und Setter fr linken Wert
    public Vector2d ValueL{
      get { return m_valL; }
      set { m_valL = value; }
   	}
    // Getter und Setter fr rechten Wert
    public Vector2d ValueR{
      get { return m_valR; }
      set { m_valR = value; }
   	}
    // Rckgabe linker Wert als Zeichenkette
    public override string StrValueL(){
      //return m_valL.ToString(); 
      return ("(" + m_valL.X.ToString() + " " + m_valL.Y.ToString() + ")");
    }
    // Rckgabe rechter Wert als Zeichenkette
    public override string StrValueR(){
      //return m_valR.ToString(); 
      return ("(" + m_valR.X.ToString() + " " + m_valR.Y.ToString() + ")");
    }
    // Test, ob beide Werte gleich sind
    public override bool EqualValues(){
      return (m_valL == m_valR);
    }
    // Rckgabe des Datentyps als (deutscher) String
    public override string typeStr(){
      return "Vektor2d";
    }
  }
 
}