using System;
using System.Xml;
using System.Xml.Schema;
using System.Collections;
using System.IO;
using System.Windows.Forms;

namespace CSysVars{

  // Die Klasse stellt alle statischen Methoden zum Einlesen und
  // Speichern von Datenstzen im XML-Format zur Verfgung
  internal class XmlUtils{

    private static string ValidationResult;
    private static bool ShowWarnings = false;


    // Speichert einen Datensatz als XML-Datei
    internal static void saveAsXml(string fileName, ArrayList values){

      // Wir nehmen erstmal UTF-8
      XmlTextWriter xWriter = new XmlTextWriter(fileName, System.Text.Encoding.UTF8);      

      // Eingerckt soll's schon sein, damit es lesbar ist
      xWriter.Formatting = Formatting.Indented;
      xWriter.Indentation = 2;
      
      // Den Header schreiben
      xWriter.WriteStartDocument(true);
      
      // Startelement mit CSysVars-Eintrag und Versionsnummer,
      // damit man feststellen kann, on die Datei wirklich kompatibel ist
      xWriter.WriteStartElement("SetOfVariables");
      xWriter.WriteAttributeString("Created-by", "CSysVars");
      xWriter.WriteAttributeString("Version", "0.1");

      for(int i = 0; i < values.Count; i++){
        SysVarData data = (SysVarData)values[i];
        xWriter.WriteStartElement(data.Label); 
        xWriter.WriteAttributeString("datatype", data.typeStr());
        if(data.typeStr() == "Zeichenkette"){
          xWriter.WriteRaw(((SysVarDataString)data).UnquotedStrValueL());
        }else{
          xWriter.WriteRaw(data.StrValueL());
        }
        xWriter.WriteEndElement();
      }

      xWriter.WriteEndElement();
      xWriter.WriteEndDocument();

      xWriter.Flush();
      xWriter.Close();
    }

    internal static XmlDocument loadXml(string fileName){
      XmlDocument xmlDoc = new XmlDocument();
      try{
        xmlDoc.Load(fileName);
      }catch(System.Exception ex){
        MessageBox.Show("Laden fehlgeschlagen!\n" + ex);
      }
      return xmlDoc;
    }

    internal static string Validate(string fileName, MemoryStream xsdValidationStream){
      XmlTextReader tReader = new XmlTextReader(fileName);

      ValidationResult = string.Empty;
      XmlValidatingReader VReader = null;
      XmlSchemaCollection xSchemaColl = new XmlSchemaCollection();
      ValidationEventHandler eventHandler = new ValidationEventHandler(ValidationCallback);

      try{

        xSchemaColl.Add(null, new XmlTextReader(xsdValidationStream));
        VReader = new XmlValidatingReader(tReader);
        VReader.Schemas.Add(xSchemaColl);
        VReader.ValidationType = ValidationType.Schema;
        VReader.ValidationEventHandler += eventHandler;
   
        while (VReader.Read()){

        }

      }catch (Exception ex){
        ex = ex;
        if ( VReader != null ){
          VReader.Close();
        }
      }
      tReader.Close();
      return ValidationResult;
    }

    public static void ValidationCallback(object sender, ValidationEventArgs args ){
      if(args.Severity == XmlSeverityType.Error){
        ValidationResult += "Error: " + args.Message + "\n";
      }else{
        if(ShowWarnings){
          ValidationResult += "Warning: " + args.Message + "\n";
        }
      }
    }

    internal static MemoryStream createXsdValidationStream(ArrayList values){
      MemoryStream stream = new MemoryStream();
      createXsdString(stream, values);
      stream.Seek(0L, SeekOrigin.Begin);
      return stream;
    }

    internal static void createXsdString(Stream stream, ArrayList values){
      XmlTextWriter xWriter = new XmlTextWriter(stream, System.Text.Encoding.UTF8);      

      xWriter.Formatting = Formatting.Indented;
      xWriter.Indentation = 2;
      
      // Den Header schreiben
      xWriter.WriteStartDocument(true);

      xWriter.WriteStartElement("xs:schema");
      xWriter.WriteAttributeString("xmlns:xs", "http://www.w3.org/2001/XMLSchema");
      xWriter.WriteAttributeString("elementFormDefault", "qualified");

      // Das root-Element
      xWriter.WriteStartElement("xs:element");
      xWriter.WriteAttributeString("name", "SetOfSysVars");
      xWriter.WriteAttributeString("type", "Set");
      xWriter.WriteEndElement();

      // Typ Set deklarieren
      xWriter.WriteStartElement("xs:complexType");
      xWriter.WriteAttributeString("name", "Set");

      xWriter.WriteStartElement("xs:sequence");

      for(int i = 0; i < values.Count; i++){
        SysVarData data = (SysVarData)values[i];
        xWriter.WriteStartElement("xs:element");
        xWriter.WriteAttributeString("name", data.Label);
        xWriter.WriteAttributeString("type", data.typeStr());
        xWriter.WriteAttributeString("minOccurs", "1");
        xWriter.WriteAttributeString("maxOccurs", "1");
        xWriter.WriteEndElement();
      }

      xWriter.WriteEndElement(); // sequence
      xWriter.WriteEndElement(); // complexType Set

      // Datentypen deklarieren
          // Integer
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Ganzzahl");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:integer");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // Double
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Fliekommazahl");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:double");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // String
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Zeichenkette");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:string");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // Boolean
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Boolean");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:string");
      xWriter.WriteStartElement("xs:pattern");
      xWriter.WriteAttributeString("value", "True|False");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // Point3d (FAKE!!!)
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Punkt3D");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:string");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // Point2d (FAKE!!!)
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Punkt2D");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:string");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // Vector3d (FAKE!!!)
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Vektor3D");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:string");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // Vector2d (FAKE!!!)
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "Vektor2D");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:string");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

          // ObjectId (fliegt raus!!!)
      xWriter.WriteStartElement("xs:simpleType");
      xWriter.WriteAttributeString("name", "ObjectId");
      xWriter.WriteStartElement("xs:restriction");
      xWriter.WriteAttributeString("base", "xs:string");
      xWriter.WriteEndElement();
      xWriter.WriteEndElement();

      xWriter.WriteEndElement(); // schema

      xWriter.Flush();
      // xWriter.Close();   ??

    }
  }

}
